import React, { forwardRef, useEffect, useMemo, useRef, useState } from 'react';
import debounce from 'lodash.debounce';
import { monaco, createEditor, getMonacoLanguage, updateEditorKeybindingsMode, registerEditorOpenHandler } from '@codingame/monaco-editor-wrapper';
import { useDeepMemo, useLastValueRef, useLastVersion, useThemeData, useUserConfiguration } from './hooks';
import './style';
const STATUS_BAR_HEIGHT = 20;
function fixCode(code) {
    // If monaco editor is provided code using \r\n
    // The editor seems to flicker and the language server is spammed by document/didChange events
    // TODO investigate me?
    return code.replace(/\r\n?/g, '\n');
}
// Some editor properties are managed directly by monaco-vim / monaco-emacs so we don't want to override them
const keyBindingsManagedOptions = {
    vim: new Set(['cursorBlinking', 'cursorWidth']),
    emacs: new Set(['cursorBlinking', 'cursorStyle']),
    classic: null
};
function removeKeyBindingsManagedOptions(options, keyBindingsMode) {
    const managedOptions = keyBindingsManagedOptions[keyBindingsMode];
    if (managedOptions == null) {
        return options;
    }
    else {
        return Object.fromEntries(Object.entries(options)
            .filter(([optionId]) => !managedOptions.has(optionId)));
    }
}
const viewStates = new Map();
export function defaultRestoreViewState(editor, model) {
    const viewState = viewStates.get(model.uri.toString());
    if (viewState != null) {
        editor.restoreViewState(viewState);
    }
}
export function defaultSaveViewState(editor, model) {
    const viewState = editor.saveViewState();
    const key = model.uri.toString();
    if (viewState != null) {
        viewStates.set(key, viewState);
    }
    else {
        viewStates.delete(key);
    }
}
function MonacoEditor({ height: requestedHeight = '100%', maxHeight, minHeight = 350, fileUri, programmingLanguage, value, options, onChange, keyBindingsMode = 'classic', markers, saveViewState = defaultSaveViewState, restoreViewState = defaultRestoreViewState, onEditorOpenRequest }, ref) {
    const editorRef = useRef();
    const modelRef = useRef();
    const preventTriggerChangeEventRef = useRef(false);
    const [height, setHeight] = useState(requestedHeight !== 'auto' ? requestedHeight : 50);
    const themeData = useThemeData();
    const containerRef = useRef(null);
    const statusBarRef = useRef(null);
    const monacoLanguage = useMemo(() => programmingLanguage != null ? getMonacoLanguage(programmingLanguage) : undefined, [programmingLanguage]);
    const userConfiguration = useUserConfiguration(monacoLanguage);
    const memoizedOptions = useDeepMemo(() => options, [options]);
    const allOptions = useMemo(() => {
        return removeKeyBindingsManagedOptions({
            ...userConfiguration,
            ...memoizedOptions,
            automaticLayout: true
        }, keyBindingsMode);
    }, [memoizedOptions, userConfiguration, keyBindingsMode]);
    const modelUri = useMemo(() => {
        return fileUri != null ? monaco.Uri.parse(fileUri) : undefined;
    }, [fileUri]);
    const fixedCode = useMemo(() => value != null ? fixCode(value) : null, [value]);
    const valueRef = useLastValueRef(fixedCode);
    const lastSaveViewState = useLastVersion(saveViewState);
    const lastRestoreViewState = useLastVersion(restoreViewState);
    // Create/Update model
    useEffect(() => {
        const value = valueRef.current;
        if (modelUri != null || value != null) {
            const existingModel = modelUri != null ? monaco.editor.getModel(modelUri) : null;
            const model = existingModel ?? monaco.editor.createModel(value, monacoLanguage, modelUri);
            if (monacoLanguage != null && model.getLanguageId() !== monacoLanguage) {
                monaco.editor.setModelLanguage(model, monacoLanguage);
            }
            modelRef.current = model;
            editorRef.current?.setModel(model);
            if (editorRef.current != null) {
                lastRestoreViewState(editorRef.current, model);
            }
            return () => {
                lastSaveViewState(editorRef.current, model);
                if (existingModel == null) {
                    // Only dispose if we are the one who created the model
                    setTimeout(() => {
                        // setTimeout is required until monaco 0.34 is released (https://github.com/TypeFox/monaco-languageclient/issues/387)
                        model.dispose();
                    });
                }
            };
        }
        else {
            modelRef.current = undefined;
            editorRef.current?.setModel(null);
        }
        return undefined;
    }, [monacoLanguage, modelUri, valueRef, lastSaveViewState, lastRestoreViewState]);
    // Create editor
    useEffect(() => {
        const containerElement = containerRef.current;
        if (containerElement != null) {
            const model = modelRef.current;
            const editor = createEditor(containerElement, {
                model,
                // We need to pass options here due to https://github.com/microsoft/monaco-editor/issues/2873
                ...allOptions,
                // We need to override all IStandaloneEditorConstructionOptions fields to prevent conflicts with proper editor options (especially `language`)
                value: undefined,
                language: undefined,
                theme: undefined,
                autoDetectHighContrast: undefined,
                accessibilityHelpUrl: undefined,
                ariaContainerElement: undefined,
                dimension: undefined,
                overflowWidgetsDomNode: undefined
            });
            editorRef.current = editor;
            if (model != null) {
                lastRestoreViewState(editor, model);
            }
            if (ref != null) {
                if (typeof ref === 'function') {
                    ref(editor);
                }
                else {
                    ref.current = editor;
                }
            }
            return () => {
                editor.dispose();
            };
        }
        return undefined;
        // eslint-disable-next-line react-hooks/exhaustive-deps
    }, []);
    // Update value
    useEffect(() => {
        if (fixedCode != null) {
            const model = modelRef.current;
            const editor = editorRef.current;
            if (fixedCode !== model.getValue()) {
                preventTriggerChangeEventRef.current = true;
                console.debug('Replacing whole editor content');
                if (editor.getOption(monaco.editor.EditorOption.readOnly)) {
                    model.setValue(fixedCode);
                }
                else {
                    editor.pushUndoStop();
                    model.pushEditOperations([], [{
                            range: model.getFullModelRange(),
                            text: fixedCode
                        }], () => null);
                    editor.pushUndoStop();
                }
                preventTriggerChangeEventRef.current = false;
            }
        }
    }, [fixedCode]);
    // Update options from props
    useEffect(() => {
        editorRef.current.updateOptions(allOptions);
    }, [allOptions]);
    // Keybindings mode
    useEffect(() => {
        const editor = editorRef.current;
        const disposable = updateEditorKeybindingsMode(editor, keyBindingsMode, statusBarRef.current);
        return () => {
            disposable.dispose();
        };
    }, [keyBindingsMode]);
    // Update markers
    useEffect(() => {
        const model = modelRef.current;
        if (markers != null && model != null) {
            monaco.editor.setModelMarkers(model, 'customMarkers', markers);
            return () => {
                monaco.editor.setModelMarkers(model, 'customMarkers', []);
            };
        }
        return undefined;
    }, [markers]);
    // Call onChange callback
    useEffect(() => {
        if (onChange != null) {
            const editor = editorRef.current;
            const didChangeModelContentDisposable = editor.onDidChangeModelContent(event => {
                if (!preventTriggerChangeEventRef.current) {
                    onChange(editor.getValue(), event);
                }
            });
            return () => {
                didChangeModelContentDisposable.dispose();
            };
        }
        return undefined;
    }, [onChange]);
    useEffect(() => {
        if (onEditorOpenRequest != null) {
            const disposable = registerEditorOpenHandler(async (model, input, source, sideBySide) => {
                if (source === editorRef.current) {
                    return onEditorOpenRequest(model, input, source, sideBySide);
                }
                return null;
            });
            return () => {
                disposable.dispose();
            };
        }
        return undefined;
    }, [onEditorOpenRequest]);
    // Compute height
    useEffect(() => {
        if (requestedHeight !== 'auto') {
            setHeight(requestedHeight);
            return;
        }
        const editor = editorRef.current;
        const recomputeHeight = () => {
            let height = Math.max(minHeight, editor.getContentHeight() + STATUS_BAR_HEIGHT);
            if (maxHeight != null) {
                height = Math.min(height, maxHeight);
            }
            setHeight(height);
        };
        const onDidContentSizeChangeDisposable = editor.onDidContentSizeChange(debounce(recomputeHeight, 50));
        recomputeHeight();
        return () => {
            onDidContentSizeChangeDisposable.dispose();
        };
    }, [minHeight, maxHeight, requestedHeight]);
    const statusBarStyle = useMemo(() => {
        return {
            backgroundColor: themeData?.getColor('statusBar.background')?.toString() ?? '#007ACC',
            color: themeData?.getColor('statusBar.foreground')?.toString() ?? '#FFFFFF',
            borderTop: `1px solid ${themeData?.getColor('statusBar.border')?.toString() ?? '#FFFFFF'}`
        };
    }, [themeData]);
    return (React.createElement("div", { className: 'react-monaco-editor-react-container', style: { height } },
        React.createElement("div", { className: 'react-monaco-editor-react-wrapper' },
            React.createElement("div", { className: 'react-monaco-editor-react', ref: containerRef }),
            React.createElement("div", { className: 'react-monaco-editor-react-status-bar', ref: statusBarRef, style: statusBarStyle }))));
}
export default forwardRef(MonacoEditor);
//# sourceMappingURL=MonacoEditor.js.map